@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho.pm 6405 2007-04-07T01:35:58.207076Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho.pm 6454 2007-04-10T02:44:06.724398Z lestrrat  $
 # 
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -6,7 +6,7 @@
 package Gungho;
 use strict;
 use warnings;
-use base qw(Class::Accessor::Fast);
+use base qw(Gungho::Base);
 use Carp qw(croak);
 use Config::Any;
 use Class::Inspector;
@@ -14,9 +14,9 @@ use UNIVERSAL::require;
 
 use Gungho::Log;
 
-__PACKAGE__->mk_accessors($_) for qw(config log provider handler engine);
+__PACKAGE__->mk_accessors($_) for qw(config log provider handler engine is_running hooks features);
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 sub new
 {
@@ -34,10 +34,36 @@ sub setup
 {
     my $self = shift;
 
+    $self->hooks({});
+    $self->features({});
+
+    $self->setup_components();
     $self->setup_log();
     $self->setup_provider();
     $self->setup_handler();
     $self->setup_engine();
+
+    $self->setup_plugins();
+
+    $self->next::method(@_); # This propagates to 
+}
+
+sub setup_components
+{
+    my $self = shift;
+
+    my $list = $self->config->{components};
+    foreach my $module (@$list) {
+        my $pkg = $self->load_gungho_module($module, 'Component');
+        $pkg->inject_base($self);
+    }
+
+    # XXX - Hack! Class::C3 doesn't like it when we have G::Base
+    # before G::Component based objects in our ISA, so remove
+    if (@$list) {
+        @Gungho::ISA = grep { $_ ne 'Gungho::Base' } @Gungho::ISA;
+        Class::C3::reinitialize();
+    }
 }
 
 sub setup_log
@@ -59,7 +85,9 @@ sub setup_provider
     }
 
     my $pkg = $self->load_gungho_module($config->{module}, 'Provider');
-    $self->provider( $pkg->new($config->{config}) );
+    my $obj = $pkg->new(config => $config->{config});
+    $obj->setup( $self );
+    $self->provider( $obj );
 }
 
 sub setup_engine
@@ -73,7 +101,9 @@ sub setup_engine
     }
 
     my $pkg = $self->load_gungho_module($config->{module}, 'Engine');
-    $self->engine( $pkg->new($config->{config}) );
+    my $obj = $pkg->new(config => $config->{config});
+    $obj->setup( $self );
+    $self->engine( $obj );
 }
 
 sub setup_handler
@@ -85,7 +115,52 @@ sub setup_handler
         config => {}
     };
     my $pkg = $self->load_gungho_module($config->{module}, 'Handler');
-    $self->handler( $pkg->new($config->{config}) );
+    my $obj = $pkg->new(config => $config->{config});
+    $obj->setup( $self );
+    $self->handler( $obj );
+}
+
+sub setup_plugins
+{
+    my $self = shift;
+
+    my $plugins = $self->config->{plugins} || [];
+    foreach my $plugin (@$plugins) {
+        my $pkg = $self->load_gungho_module($plugin->{module}, 'Plugin');
+        my $obj = $pkg->new(config => $plugin->{config});
+        $obj->setup($self);
+    }
+}
+
+sub register_hook
+{
+    my $self = shift;
+    my $hooks = $self->hooks;
+    while(@_) {
+        my($name, $hook) = splice(@_, 0, 2);
+        $hooks->{$name} ||= [];
+        push @{ $hooks->{$name} }, $hook;
+    }
+}
+
+sub run_hook
+{
+    my $self = shift;
+    my $name = shift;
+    my $hooks = $self->hooks->{$name} || [];
+    foreach my $hook (@{ $hooks }) {
+        if (ref($hook) eq 'CODE') {
+            $hook->($self, @_);
+        } else {
+            $hook->execute($self, @_);
+        }
+    }
+}
+
+sub has_feature
+{
+    my ($self, $name) = @_;
+    return exists $self->features()->{$name};
 }
 
 sub load_config
@@ -126,24 +201,35 @@ sub load_gungho_module
     return $pkg;
 }
 
-sub run { $_[0]->engine->run($_[0]) }
+sub run
+{
+    $_[0]->is_running(1);
+    $_[0]->engine->run($_[0]);
+}
 
 sub has_requests
 {
-    my $self = shift;
-    $self->provider->has_requests;
+    my ($self) = @_;
+    $self->provider->has_requests(@_);
 }
 
 sub get_requests
 {
-    my $self = shift;
-    $self->provider->get_requests;
+    my ($self) = @_;
+    $self->provider->get_requests(@_);
+}
+
+sub send_request
+{
+    my ($self) = @_;
+$self->log->debug("Sending request!");
+    $self->engine->send_request(@_);
 }
 
 sub handle_response
 {
-    my ($self, $response) = @_;
-    $self->handler->handle_response($self, $response);
+    my ($self) = @_;
+    $self->handler->handle_response(@_);
 }
 
 1;
@@ -166,6 +252,9 @@ Xango was *fast*, but it was horribly hard to debug. Gungho tries to build
 from clean structures, based upon principles from the likes of Catalyst and
 Plagger.
 
+All components (engine, provider, handler) are overridable and switcheable.
+Plugin mechanism is available to add hooks to be executed during the run.
+
 WARNING: *ALL* APIs are still subject to change.
 
 =head1 STRUCTURE
@@ -174,6 +263,18 @@ Gungho is comprised of three parts. A Provider, which provides Gungho with
 requests to process, a Handler, which handles the fetched page, and an
 Engine, which controls the entire process.
 
+There are also "hooks". These hooks can be registered from anywhere by
+invoking the register_hook() method. They are run at particular points,
+which are specified when you call register_hook().
+
+=head1 HOOKS
+
+Currently available hooks are:
+
+=head2 engine.send_request
+
+=head2 engine.handle_response
+
 =head1 METHODS
 
 =head2 new($config)
@@ -185,21 +286,37 @@ or a hashref.
 
 Starts the Gungho process.
 
-=head2 setup
+=head2 has_feature($name)
+
+Returns true if Gungho supports some feature $name
+
+=head2 setup()
 
 Sets up the Gungho environment, including calling the various setup_*
 methods to configure the provider, engine, handler, etc.
 
-=head2 setup_engine
+=head2 setup_components()
 
-=head2 setup_handler
+=head2 setup_engine()
 
-=head2 setup_log
+=head2 setup_handler()
 
-=head2 setup_provider
+=head2 setup_log()
+
+=head2 setup_provider()
+
+=head2 setup_plugins()
 
 Sets up the various components.
 
+=head2 register_hook($hook_name => $coderef[, $hook_name => $coderef])
+
+Registers a hook to be run under the specified $hook_name
+
+=head2 run_hook($hook_name)
+
+Runs all the hooks under the hook $hook_name
+
 =head2 has_requests
 
 Delegates to provider's has_requests
@@ -212,6 +329,10 @@ Delegates to provider's get_requests
 
 Delegates to handler's handle_response
 
+=head2 send_request
+
+Delegates to engine's send_request
+
 =head2 load_config($config)
 
 Loads the config from $config via Config::Any.
